@@ -32,13 +32,11 @@ from spotPython.utils.progress import progress_bar
 from spotPython.utils.convert import find_indices
 from spotPython.hyperparameters.values import (
     get_bound_values,
-    get_fun_control_fun_evals,
-    get_fun_control_fun_repeats,
-    set_fun_control_seed,
-    set_fun_control_sigma,
+    set_fun_control_key_value,
+    get_fun_control_key_value,
 )
 import plotly.graph_objects as go
-from typing import List, Union, Callable, Dict
+from typing import Union, Callable, Dict
 
 
 logger = logging.getLogger(__name__)
@@ -202,13 +200,12 @@ class Spot:
         upper: np.array = None,
         fun_evals: int = 15,
         fun_repeats: int = 1,
-        fun_control: Dict[str, Union[int, float]] = {},
+        fun_control: dict = {},
         max_time: int = inf,
         noise: bool = False,
         tolerance_x: float = 0,
-        var_type: List[str] = ["num"],
-        var_name: List[str] = None,
-        all_var_name: List[str] = None,
+        var_type: list = ["num"],
+        var_name: list = None,
         infill_criterion: str = "y",
         n_points: int = 1,
         ocba_delta: int = 0,
@@ -225,79 +222,85 @@ class Spot:
         optimizer_control: Dict[str, Union[int, float]] = {},
     ):
         self.fun = fun
-        # if fun is None, raise an exception
         if self.fun is None:
             raise Exception("No objective function specified.")
-        # if fun is not callable, raise an exception
         if not callable(self.fun):
             raise Exception("Objective function is not callable.")
+
+        # 1. fun_control updates:
+        # -----------------------
         self.fun_control = fun_control
+
         # set fun_control["sigma"] to sigma if "sigma" is not in fun_control dictionary
-        if "sigma" not in self.fun_control:
-            set_fun_control_sigma(fun_control, sigma)
-        # set fun_control["seed"] to seed if "seed" is not in fun_control dictionary
-        if "seed" not in self.fun_control:
-            set_fun_control_seed(self.fun_control, seed)
-        self.lower = lower
+        # If no fun_control is provided, the fun_control dictionary is initialized with the
+        # key "sigma" and the value sigma amd the key "seed" and the value seed.
+        # This is a mimimum requirement for the fun_control dictionary to call
+        # objective functions from the spotPython.fun.objectivefunctions module.
+        set_fun_control_key_value(fun_control=self.fun_control, key="sigma", value=sigma)
+        set_fun_control_key_value(fun_control=self.fun_control, key="seed", value=seed)
+
+        # 2. self attributes updates:
+        # -----------------------
         # if lower is in the fun_control dictionary, use the value of the key "lower" as the lower bound
         # else use the lower bound lower
-        if get_bound_values(fun_control, "lower") is not None:
-            self.lower = get_bound_values(fun_control, "lower")
+        self.lower = lower
+        if get_bound_values(self.fun_control, "lower") is not None:
+            self.lower = get_bound_values(self.fun_control, "lower")
+        # Number of dimensions is based on lower
+        self.k = self.lower.size
+
         # if upper is in fun_control dictionary, use the value of the key "upper" as the upper bound
         # else use the upper bound upper
         self.upper = upper
-        if get_bound_values(fun_control, "upper") is not None:
-            self.upper = get_bound_values(fun_control, "upper")
-        self.var_type = var_type
-        self.var_name = var_name
-        # use x0, x1, ... as default variable names:
-        if self.var_name is None:
-            self.var_name = ["x" + str(i) for i in range(len(self.lower))]
-        self.all_var_name = all_var_name
-        # Reduce dim based on lower == upper logic:
-        # modifies lower, upper, and var_type
-        self.to_red_dim()
-        self.k = self.lower.size
+        if get_bound_values(self.fun_control, "upper") is not None:
+            self.upper = get_bound_values(self.fun_control, "upper")
+
+        self.set_self_attribute("var_type", var_type, self.fun_control)
         # Force numeric type as default in every dim:
         # assume all variable types are "num" if "num" is
-        # specified once:
+        # specified less than k times
         if len(self.var_type) < self.k:
             self.var_type = self.var_type * self.k
             logger.warning("All variable types forced to 'num'.")
-        # if fun_evals is in fun_control dictionary,
-        # use the value of the key "fun_evals" as the number of function evaluations
-        self.fun_evals = fun_evals
-        if get_fun_control_fun_evals(fun_control) is not None:
-            self.fun_evals = get_fun_control_fun_evals(fun_control)
-        # if fun_repeats is in fun_control dictionary,
-        # use the value of the key "fun_repeats" as the number of function repeats
-        # else use the number of function repeats fun_repeats
-        self.fun_repeats = fun_repeats
-        if get_fun_control_fun_repeats(fun_control) is not None:
-            self.fun_repeats = get_fun_control_fun_repeats(fun_control)
-        self.max_time = max_time
-        self.noise = noise
-        self.tolerance_x = tolerance_x
-        self.ocba_delta = ocba_delta
-        self.log_level = log_level
-        self.show_models = show_models
-        self.show_progress = show_progress
+
+        self.set_self_attribute("var_name", var_name, self.fun_control)
+        # use x0, x1, ... as default variable names:
+        if self.var_name is None:
+            self.var_name = ["x" + str(i) for i in range(len(self.lower))]
+
+        # Reduce dim based on lower == upper logic:
+        # modifies lower, upper, var_type, and var_name
+        self.to_red_dim()
+
+        self.set_self_attribute("fun_evals", fun_evals, self.fun_control)
+        self.set_self_attribute("fun_repeats", fun_repeats, self.fun_control)
+        self.set_self_attribute("max_time", max_time, self.fun_control)
+        self.set_self_attribute("noise", noise, self.fun_control)
+        self.set_self_attribute("tolerance_x", tolerance_x, self.fun_control)
+        self.set_self_attribute("ocba_delta", ocba_delta, self.fun_control)
+        self.set_self_attribute("log_level", log_level, self.fun_control)
+        self.set_self_attribute("show_models", show_models, self.fun_control)
+        self.set_self_attribute("show_progress", show_progress, self.fun_control)
+        self.set_self_attribute("infill_criterion", infill_criterion, self.fun_control)
+        self.set_self_attribute("n_points", n_points, self.fun_control)
+
         # Random number generator:
         self.rng = default_rng(self.fun_control["seed"])
-        self.infill_criterion = infill_criterion
-        # Bounds
+
+        # Bounds are internal, because they are functions of self.lower and self.upper
+        # and used by the optimizer:
         de_bounds = []
-        for j in range(self.k):
+        for j in range(self.lower.size):
             de_bounds.append([self.lower[j], self.upper[j]])
         self.de_bounds = de_bounds
-        # Infill points:
-        self.n_points = n_points
+
         # Design related information:
         self.design = design
         if design is None:
-            self.design = spacefilling(k=self.k, seed=self.fun_control["seed"])
+            self.design = spacefilling(k=self.lower.size, seed=self.fun_control["seed"])
         self.design_control = {"init_size": 10, "repeats": 1}
         self.design_control.update(design_control)
+
         # Surrogate related information:
         self.surrogate = surrogate
         self.surrogate_control = {
@@ -331,7 +334,7 @@ class Spot:
 
         # if the key "spot_writer" is not in the dictionary fun_control,
         # set self.spot_writer to None else to the value of the key "spot_writer"
-        self.spot_writer = fun_control.get("spot_writer", None)
+        self.spot_writer = self.fun_control.get("spot_writer", None)
         self.surrogate_control.update(surrogate_control)
         # If no surrogate model is specified, use the internal
         # spotPython kriging surrogate:
@@ -363,6 +366,22 @@ class Spot:
         logger.debug("In Spot() init(): fun_control: %s", self.fun_control)
         logger.debug("In Spot() init(): optimizer_control: %s", self.optimizer_control)
         logger.debug("In Spot() init(): surrogate_control: %s", self.surrogate_control)
+        logger.debug("In Spot() init(): self.get_spot_attributes_as_df(): %s", self.get_spot_attributes_as_df())
+
+    def set_self_attribute(self, attribute, value, dict):
+        """
+        This function sets the attribute of the 'self' object to the provided value.
+        If the key exists in the provided dictionary, it updates the attribute with the value from the dictionary.
+
+        Args:
+            self (object): the object whose attribute is to be set
+            attribute (str): the attribute to set
+            value (Any): the value to set the attribute to
+            dict (dict): the dictionary to check for the key
+        """
+        setattr(self, attribute, value)
+        if get_fun_control_key_value(fun_control=dict, key=attribute) is not None:
+            setattr(self, attribute, get_fun_control_key_value(fun_control=dict, key=attribute))
 
     def get_spot_attributes_as_df(self) -> pd.DataFrame:
         """Get all attributes of the spot object as a pandas dataframe.
@@ -445,14 +464,73 @@ class Spot:
         return df
 
     def to_red_dim(self) -> None:
+        """
+        Reduce dimension if lower == upper.
+        This is done by removing the corresponding entries from
+        lower, upper, var_type, and var_name.
+        k is modified accordingly.
+
+        Args:
+            self (object): Spot object
+
+        Returns:
+            (NoneType): None
+
+        Attributes:
+            self.lower (numpy.ndarray): lower bound
+            self.upper (numpy.ndarray): upper bound
+            self.var_type (List[str]): list of variable types
+
+        Examples:
+            >>> import numpy as np
+                from spotPython.fun.objectivefunctions import analytical
+                from spotPython.spot import spot
+                # number of initial points:
+                ni = 3
+                # number of points
+                n = 10
+                fun = analytical().fun_sphere
+                lower = np.array([-1, -1])
+                upper = np.array([1, 1])
+                design_control={"init_size": ni}
+                spot_1 = spot.Spot(fun=fun,
+                            lower = lower,
+                            upper= upper,
+                            fun_evals = n,
+                            show_progress=True,
+                            design_control=design_control,)
+                spot_1.run()
+                assert spot_1.lower.size == 2
+                assert spot_1.upper.size == 2
+                assert len(spot_1.var_type) == 2
+                assert spot_1.red_dim == False
+                spot_1.lower = np.array([-1, -1])
+                spot_1.upper = np.array([-1, -1])
+                spot_1.to_red_dim()
+                assert spot_1.lower.size == 0
+                assert spot_1.upper.size == 0
+                assert len(spot_1.var_type) == 0
+                assert spot_1.red_dim == True
+
+        """
+        # Backup of the original values:
         self.all_lower = self.lower
         self.all_upper = self.upper
+        # Select only lower != upper:
         self.ident = (self.upper - self.lower) == 0
+        # Determine if dimension is reduced:
+        self.red_dim = self.ident.any()
+        # Modifications:
+        # Modify lower and upper:
         self.lower = self.lower[~self.ident]
         self.upper = self.upper[~self.ident]
-        self.red_dim = self.ident.any()
-        self.all_var_type = self.var_type
-        self.var_type = [x for x, y in zip(self.all_var_type, self.ident) if not y]
+        # Modify k (dim):
+        self.k = self.lower.size
+        # Modify var_type:
+        if self.var_type is not None:
+            self.all_var_type = self.var_type
+            self.var_type = [x for x, y in zip(self.all_var_type, self.ident) if not y]
+        # Modify var_name:
         if self.var_name is not None:
             self.all_var_name = self.var_name
             self.var_name = [x for x, y in zip(self.all_var_name, self.ident) if not y]
@@ -652,7 +730,10 @@ class Spot:
         self.X = X0
         # (S-3): Eval initial design:
         X_all = self.to_all_dim_if_needed(X0)
+        logger.debug("In Spot() initialize_design(), before calling self.fun: X_all: %s", X_all)
+        logger.debug("In Spot() initialize_design(), before calling self.fun: fun_control: %s", self.fun_control)
         self.y = self.fun(X=X_all, fun_control=self.fun_control)
+        logger.debug("In Spot() initialize_design(), after calling self.fun: self.y: %s", self.y)
         # TODO: Error if only nan values are returned
         logger.debug("New y value: %s", self.y)
         #
